% Invertible Syntax Descriptions: Unifying Parsing and Pretty Printing
% Tillmann Rendel\and Klaus Ostermann

\date{}
\institute{Marburg University\\[4em] Haskell Symposium, September 30, 2010}

Title Page
----------

Parsing and Pretty Printing
---------------------------

 * parser combinator libraries
 * pretty printing libraries
  
< data Exp = ...
< 
< parseExp  ::  Parser Exp
< printExp  ::  Exp -> Doc

%include AST
%include OnlyParsing
%include OnlyPrinting
%include Syntax

Unifying Choice
---------------
 
< (<|>) :: Parser alpha -> Parser alpha -> Parser alpha

\vspace{0.4em}

< type Printer alpha = alpha -> Maybe String
< (<|>) :: Printer alpha -> Printer alpha -> Printer alpha

\vspace{0.4em}

< (<|>) :: Syntax delta => delta alpha -> delta alpha -> delta alpha 

Unifying Mapping
----------------

< (<$>) :: (alpha -> beta) -> (Parser alpha -> Parser beta)

\vspace{0.4em}
                       
< (<$>) :: (beta -> alpha) -> (Printer alpha -> Printer beta)

\vspace{0.4em}

< (<$>) :: Syntax delta => Iso alpha beta -> (delta alpha -> delta beta) 

Partial Isomorphisms
--------------------

< data Iso alpha beta
<   = Iso  (alpha  ->  Maybe  beta)
<          (beta   ->  Maybe  alpha)
 
%format <==> = "\quad\Leftrightarrow\quad"
 
 * `f x == Just y <==> g y == Just x`
 
Constructors
------------

< app :: Iso (Exp, Exp) Exp
< app = Iso f g where
<   f (a, b) = Just (App a b)
< 
<   g (App a b)  =  Just (a, b)
<   g _          =  Nothing

 * Template Haskell macro

Folding
-------

< foldl :: (alpha -> beta -> alpha) -> alpha -> [beta] -> alpha

\vspace{0.4em}

< foldl :: Iso (alpha, beta) alpha -> Iso (alpha, [beta]) alpha

\pause

 * `inverse foldl` is `unfoldl`! 

Unified Syntax Description
--------------------------

< syntaxSK = exp1 where
<   exp0   =                           {-"\alert<2>{"-}s{-"}"-}  {-"\alert<4>{"-}<$>{-"}"-}  tok "S"
<          {-"\alert<3>{"-}<|>{-"}"-}  {-"\alert<2>{"-}k{-"}"-}  {-"\alert<4>{"-}<$>{-"}"-}  tok "K"
<          {-"\alert<3>{"-}<|>{-"}"-}  tok "(" *> exp1 <* tok ")"
<   
<   exp1   =    {-"\alert<5>{"-}foldl{-"}"-} {-"\alert<2>{"-}app{-"}"-} {-"\alert<4>{"-}<$>{-"}"-} exp0 <*> many exp0
<   
<   tok k  =    string k <* spaces

Unfolding
---------

\begin{postit}
\hfill
\visible<4>{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{S}$};
\end{tikzpicture}}
\hfill
\visible<4>{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{K}$};
\end{tikzpicture}}
\visible<3>{\llap{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{App}$}
  child {node {$\Varid{S}$}}
  child {node {$\Varid{K}$}};
\end{tikzpicture}}}
\hfill
\visible<3->{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{K}$};
\end{tikzpicture}}
\visible<2>{\llap{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{App}$} 
    child {node {$\Varid{App}$}
      child {node {$\Varid{S}$}}
      child {node {$\Varid{K}$}}}
    child {node {$\Varid{K}$}};
\end{tikzpicture}}}
\hfill
\visible<2->{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{App}$}
    child {node {$\Varid{App}$}
      child {node {$\Varid{S}$}}
      child {node {$\Varid{K}$}}}
    child {node {$\Varid{K}$}};
\end{tikzpicture}}
\visible<1>{\llap{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{App}$} 
  child {node {$\Varid{App}$}
    child {node {$\Varid{App}$}
      child {node {$\Varid{S}$}}
      child {node {$\Varid{K}$}}}
    child {node {$\Varid{K}$}}}
  child {node {$\Varid{App}$}
    child {node {$\Varid{App}$}
      child {node {$\Varid{S}$}}
      child {node {$\Varid{K}$}}}
    child {node {$\Varid{K}$}}};
\end{tikzpicture}}}
\hfill
\end{postit}

Unified Syntax Description
--------------------------

< syntaxSK = exp1 where
<   exp0   =                           {-"\alert<2>{"-}s{-"}"-}  {-"\alert<4>{"-}<$>{-"}"-}  tok "S"
<          {-"\alert<3>{"-}<|>{-"}"-}  {-"\alert<2>{"-}k{-"}"-}  {-"\alert<4>{"-}<$>{-"}"-}  tok "K"
<          {-"\alert<3>{"-}<|>{-"}"-}  tok "(" *> exp1 <* tok ")"
<   
<   exp1   =    {-"\alert<5>{"-}foldl{-"}"-} {-"\alert<2>{"-}app{-"}"-} {-"\alert<4>{"-}<$>{-"}"-} exp0 <*> many exp0
<   
<   tok k  =    string k <* spaces

Atomic terms
------------

\begin{postit}
\hfill
\alt<1>{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{S}$};
\end{tikzpicture}}{$\text{\tt\char34 S\char34}$}
\hfill
\alt<-2>{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{K}$};
\end{tikzpicture}}{$\text{\tt\char34 K\char34}$}
\hfill
\alt<-3>{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{K}$};
\end{tikzpicture}}{$\text{\tt\char34 S\char34}$}
\hfill
\visible<-4>{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{App}$}
    child {node {$\Varid{App}$}
      child {node {$\Varid{S}$}}
      child {node {$\Varid{K}$}}}
    child {node {$\Varid{K}$}};
\end{tikzpicture}}
\hfill
\end{postit}

Unified Syntax Description
--------------------------

< syntaxSK = exp1 where
<   exp0   =                           {-"\alert<2>{"-}s{-"}"-}  {-"\alert<4>{"-}<$>{-"}"-}  tok "S"
<          {-"\alert<3>{"-}<|>{-"}"-}  {-"\alert<2>{"-}k{-"}"-}  {-"\alert<4>{"-}<$>{-"}"-}  tok "K"
<          {-"\alert<3>{"-}<|>{-"}"-}  tok "(" *> exp1 <* tok ")"
<   
<   exp1   =    {-"\alert<5>{"-}foldl{-"}"-} {-"\alert<2>{"-}app{-"}"-} {-"\alert<4>{"-}<$>{-"}"-} exp0 <*> many exp0
<   
<   tok k  =    string k <* spaces

Recursive call
--------------

\begin{postit}
\hfill
$\text{\tt\char34 S\char34}$
\hfill
$\text{\tt\char34 K\char34}$
\hfill
$\text{\tt\char34 S\char34}$
\hfill
\alt<2>{$\text{\tt\char34 (SKK) \char34}$\vphantom{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{App}$}
    child {node {$\Varid{App}$}
      child {node {$\Varid{S}$}}
      child {node {$\Varid{K}$}}}
    child {node {$\Varid{K}$}};
\end{tikzpicture}}}{\begin{tikzpicture}
  [level 1/.style={sibling distance=4em, level distance=2em},
   level 2/.style={sibling distance=2em, level distance=2em},
   level 3/.style={sibling distance=2em, level distance=2em},
   baseline=(root.base)]
\node (root) {$\Varid{App}$}
    child {node {$\Varid{App}$}
      child {node {$\Varid{S}$}}
      child {node {$\Varid{K}$}}}
    child {node {$\Varid{K}$}};
\end{tikzpicture}}
\hfill
\end{postit}

In the paper
------------

 * Proof-of-concept Implementation
 * Algebra of Partial Isomorphisms
 * Operator Priorities
 * Derivation of `foldl`
 * Related Work
 * Full Code
 
Future Work
-----------

 * `Syntax` instances for existing libraries
 * Larger case-study
 * proof-carrying partial isomorphisms
 * proof of round-trip property
 
Haskell wish list
-----------------

 *  Better support for `Category`
   
    (`Functor` etc. between different categories)
    
 *  Arrow notation for nearly-arrows
    
    (no `arr :: (alpha -> beta) -> (Iso alpha beta)`)

 * Generic isomorphisms for constructors   
 
Summary
-------

 * unified combinator library for parsing and pretty printing
 * based on functor from partial isomorphisms
 * looks like existing parser combinator parsers
 * pretty printing for free!

\textbf{\only<2>{\hfill Thank you!}}
 
Invertible Folding
------------------

 * Entry point

%format *** = "\times" 
 
< foldl :: Iso (alpha, beta) alpha -> Iso (alpha, [beta]) alpha
< foldl i  =  inverse unit 
<          .  (id *** inverse nil) 
<          .  {-"\alert<2>{"-} iterate (step i) {-"}"-}

Invertible Folding
------------------

 * Invertible step function 

< step i  =  ({-"\alert<2>{"-} i {-"}"-} *** id) 
<         .  associate 
<         .  (id *** {-"\alert<2>{"-} inverse cons {-"}"-})

 * Exchange top-level `i` and `cons`

Invertible Folding
------------------

 * Iteration of partial isomorphisms

< iterate :: Iso alpha alpha -> Iso alpha alpha
< iterate step = Iso f g where
<   f  =  Just  .  driver  (apply    step)
<   g  =  Just  .  driver  (unapply  step)

Invertible Folding
------------------

 * Iteration of partial functions

< driver :: (alpha -> Maybe alpha) -> (alpha -> alpha)
< driver step state 
<   =  case step state of
<        Just state'  ->  driver step state'
<        Nothing      ->  state

Type classes
------------

< class IsoFunctor f where
<   (<$>)  ::  Iso alpha beta -> (f alpha -> f beta)
<
< class Alternative f where
<   (<|>) :: f alpha -> f alpha -> f alpha
<   empty :: f alpha
<
< class ProductFunctor f where
<   (<*>)  ::  f alpha -> f beta -> f (alpha, beta)

Type classes
------------

< class (  IsoFunctor delta, 
<          ProductFunctor delta, 
<          Alternative delta) 
<    => Syntax delta where
<   pure   ::  Eq alpha => alpha -> delta alpha
<   token  ::  delta Char

Parser Implementation
---------------------

< newtype Parser alpha 
<   = Parser (String -> [(alpha, String)])
<
< parse :: Parser alpha -> String -> [alpha]
< parse (Parser p) s = [ x | (x, "") <- p s ]

Parser Implementation
---------------------

< instance IsoFunctor Parser where
<   iso <$> Parser p 
<     = Parser (\s ->  [  (y, s') 
<                      |  (x, s')  <-  p s
<                      ,  Just y   <-  [apply iso x] ])

Parser Implementation
---------------------

< instance ProductFunctor Parser where
<   Parser p <*> Parser q 
<     = Parser (\s ->  [  ((x, y), s'') 
<                      |  (x,  s')   <- p  s
<                      ,  (y,  s'')  <- q  s' ])

Parser Implementation
---------------------

< instance Alternative Parser where
<   Parser p <|> Parser q 
<     = Parser (\s -> p s ++ q s)
<   empty = Parser (\s -> [])

Parser Implementation
---------------------

< instance Syntax Parser where
<   pure x  =  Parser (\s -> [(x, s)])
<   token   =  Parser f where
<     f []      =  []
<     f (t:ts)  =  [(t, ts)] 

Printer Implementation
----------------------

< newtype Printer alpha = Printer (alpha -> Maybe String)
< 
< print :: Printer alpha -> alpha -> Maybe String
< print (Printer p) x = p x

Printer Implementation
----------------------

< instance IsoFunctor Printer where
<   iso <$> Printer p 
<     = Printer (\b -> unapply iso b >>= p)

Printer Implementation
----------------------

< instance ProductFunctor Printer where
<   Printer p <*> Printer q 
<     = Printer (\(x, y) -> liftM2 (++) (p x) (q y))

Printer Implementation
----------------------

< instance Alternative Printer where
<   Printer p <|> Printer q 
<     = Printer (\s -> mplus (p s) (q s))
<   empty = Printer (\s -> Nothing)

Printer Implementation
----------------------

< instance Syntax Printer where
<   pure x = Printer (\y ->  if x == y 
<                              then Just "" 
<                              else Nothing) 
<   token = Printer (\t -> Just [t])

